import {
  timeParse,
  utcWeek,
  utcWeeks,
  group,
  stack,
  extent,
  stackOffsetWiggle,
  stackOrderInsideOut,
  descending,
} from 'd3';
import { blur } from 'array-blur';

// The max number of layers.
const topN = 100;

const parseDate = timeParse('%Y-%m-%d');

// These projects are not really "Stamen Projects" per se.
// Stamen just so happened to fork them.
const blacklist = new Set([
  'mapbox-gl-js',
  'openmaptiles',
  'xyz-showcase',
  'WhirlyGlobe',
  'react-fullpage',
  'react-dropdown',
  'react-autosuggest',
  'react-slick',
  'natural-earth-vector',
  'react-typeahead',
  'openstreetmap-carto',
  'intro.js',
  'leaflet-rails',
  'osm',
  'GetOutside',
  'Cinder-MsKinect',
  'tm2',
]);

//const layer = (d) => d.repo;
const layer = (d) => d.author;

export const transformData = (data) => {
  data.forEach((d) => {
    d.date = utcWeek.floor(parseDate(d.date.split(' ')[0]));
  });

  // Aggregate by month and repository.
  const groupedData = group(data, (d) => d.date, layer);

  const layerGroupedData = group(data, layer);

  const layers = Array.from(layerGroupedData.keys()).filter(
    (layer) => !blacklist.has(layer)
  );

  const [start, stop] = extent(data, (d) => d.date);
  const allWeeks = utcWeeks(start, stop);

  const dataByLayer = new Map();

  for (let layer of layers) {
    let sum = 0;
    const layerData = allWeeks.map((date) => {
      const value = groupedData.get(date);
      const commits = value ? value.get(layer) : null;
      const commitCount = commits ? commits.length : 0;
      sum += commitCount;
      return commitCount;
    });

    // Apply smoothing
    const smoothedLayerData = blur().radius(13)(layerData);

    smoothedLayerData.sum = sum;
    smoothedLayerData.layer = layer;

    dataByLayer.set(layer, smoothedLayerData);
  }

  // Show only the top N,
  // to avoid performance problems that come
  // with too many layers.
  const visibleLayers = layers
    .map((layer) => dataByLayer.get(layer))
    .sort((a, b) => descending(a.sum, b.sum))
    .slice(0, topN)
    .map((d) => d.layer);

  const transformedData = [];
  allWeeks.forEach((date, i) => {
    const row = { date };
    for (let layer of visibleLayers) {
      row[layer] = dataByLayer.get(layer)[i];
    }
    transformedData.push(row);
  });

  const stackedData = stack()
    .offset(stackOffsetWiggle)
    .order(stackOrderInsideOut)
    .keys(visibleLayers)(transformedData);

  return { data, stackedData };
};
